
def GCD(a,b):
    if a < b:
        a,b = b,a
    if b < 0:
        b *= -1
    while b > 0:
        a,b = b,a%b
    return a

#
# This example uses Python's special methods which are documented here
# https://docs.python.org/3/reference/datamodel.html#special-method-names
#
class Rational:

    def __init__(self,n,d):
        if d < 0:
            n *= -1
            d *= -1
        g = GCD(n,d)
        self.num = n // g
        self.den = d // g

    def numerator(self):
        return self.num

    def denominator(self):
        return self.den

    # Defines what == means.
    def __eq__(self,other):
        ns = self.numerator()
        ds = self.denominator()
        no = other.numerator()
        do = other.denominator()
        return (ns * do == no * ds)

    # Defines what print prints. (And conversion using str)
    def __str__(self):
        n = self.numerator()
        d = self.denominator()
        ntext = str(n)
        dtext = str(d)
        if d == 1:
            return ntext
        else:
            return ntext + "/" + dtext

    # Defines what + means.
    def __add__(self,other):
        ns = self.numerator()
        ds = self.denominator()
        no = other.numerator()
        do = other.denominator()
        return Rational(ns*do + no*ds, ds*do)

    # Defines what * means. 
    def __mul__(self,other):
        ns = self.numerator()
        ds = self.denominator()
        no = other.numerator()
        do = other.denominator()
        return Rational(ns*no, ds*do)

    # Defines what gets displayed within the interpreter.
    # In class I just had this:
    #
    # __repr__ = __str__
    #
    # But the Python documentation would rather have me do this:
    def __repr__(self):
        n = self.numerator()
        d = self.denominator()
        return f"Rational({repr(n)},{repr(d)})"

    # Defines what [] means.
    def __getitem__(self,which):
        if which == 0:
            return self.numerator();
        elif which == 1:
            return self.denominator();
        else:
            raise Exception('Invalid index.')
